import logging
from typing import Dict, Any, List, Optional
from datetime import datetime, timedelta
from telegram import Update
from telegram.ext import ContextTypes
from .base import InfoCommandsBase

logger = logging.getLogger(__name__)

class StatsCommands(InfoCommandsBase):
    """Handles all statistics-related commands."""

    async def stats_command(self, update: Update, context: ContextTypes.DEFAULT_TYPE) -> None:
        """Handle the /stats command."""
        try:
            if not self._is_authorized(update):
                await self._reply(update, "❌ Unauthorized access.")
                return

            stats = self.trading_engine.get_stats()
            if not stats:
                await self._reply(update, "❌ Trading stats not available.")
                return

            # Get trading stats
            trading_stats = stats.get_trading_stats()
            if not trading_stats:
                await self._reply(update, "❌ Trading statistics not available.")
                return

            # Format stats text
            stats_text = "📊 <b>Trading Statistics</b>\n\n"

            # Add total trades
            total_trades = trading_stats.get('total_trades', 0)
            stats_text += f"📈 Total Trades: {total_trades}\n"

            # Add winning trades
            winning_trades = trading_stats.get('winning_trades', 0)
            win_rate = (winning_trades / total_trades * 100) if total_trades > 0 else 0
            stats_text += f"✅ Winning Trades: {winning_trades} ({win_rate:.2f}%)\n"

            # Add losing trades
            losing_trades = trading_stats.get('losing_trades', 0)
            loss_rate = (losing_trades / total_trades * 100) if total_trades > 0 else 0
            stats_text += f"❌ Losing Trades: {losing_trades} ({loss_rate:.2f}%)\n"

            # Add total P&L
            total_pnl = trading_stats.get('total_pnl', 0.0)
            pnl_emoji = "🟢" if total_pnl >= 0 else "🔴"
            stats_text += f"{pnl_emoji} Total P&L: ${total_pnl:,.2f}\n"

            # Add average P&L per trade
            avg_pnl = total_pnl / total_trades if total_trades > 0 else 0
            avg_pnl_emoji = "🟢" if avg_pnl >= 0 else "🔴"
            stats_text += f"{avg_pnl_emoji} Average P&L per Trade: ${avg_pnl:,.2f}\n"

            # Add ROE metrics
            total_roe = trading_stats.get('total_roe', 0.0)
            avg_roe = trading_stats.get('average_roe', 0.0)
            best_roe = trading_stats.get('best_roe', 0.0)
            worst_roe = trading_stats.get('worst_roe', 0.0)
            best_roe_token = trading_stats.get('best_roe_token', 'N/A')
            worst_roe_token = trading_stats.get('worst_roe_token', 'N/A')

            roe_emoji = "🟢" if total_roe >= 0 else "🔴"
            stats_text += f"{roe_emoji} Total ROE: {total_roe:+.2f}%\n"
            stats_text += f"{roe_emoji} Average ROE per Trade: {avg_roe:+.2f}%\n"
            stats_text += f"🏆 Best ROE: {best_roe:+.2f}% ({best_roe_token})\n"
            stats_text += f"💔 Worst ROE: {worst_roe:+.2f}% ({worst_roe_token})\n"

            # Add largest win and loss
            largest_win = trading_stats.get('largest_win', 0.0)
            largest_loss = trading_stats.get('largest_loss', 0.0)
            stats_text += f"🏆 Largest Win: ${largest_win:,.2f}\n"
            stats_text += f"💔 Largest Loss: ${largest_loss:,.2f}\n"

            # Add average win and loss
            avg_win = trading_stats.get('average_win', 0.0)
            avg_loss = trading_stats.get('average_loss', 0.0)
            stats_text += f"📈 Average Win: ${avg_win:,.2f}\n"
            stats_text += f"📉 Average Loss: ${avg_loss:,.2f}\n"

            # Add profit factor
            profit_factor = trading_stats.get('profit_factor', 0.0)
            stats_text += f"⚖️ Profit Factor: {profit_factor:.2f}\n"

            # Add time-based stats
            first_trade_time = trading_stats.get('first_trade_time')
            if first_trade_time:
                try:
                    first_trade = datetime.fromisoformat(first_trade_time.replace('Z', '+00:00'))
                    trading_duration = datetime.now(first_trade.tzinfo) - first_trade
                    days = trading_duration.days
                    hours = trading_duration.seconds // 3600
                    stats_text += f"⏱️ Trading Duration: {days}d {hours}h\n"
                except ValueError:
                    logger.warning(f"Could not parse first_trade_time: {first_trade_time}")

            # Add trades per day
            if first_trade_time:
                try:
                    first_trade = datetime.fromisoformat(first_trade_time.replace('Z', '+00:00'))
                    trading_duration = datetime.now(first_trade.tzinfo) - first_trade
                    days = max(trading_duration.days, 1)  # Avoid division by zero
                    trades_per_day = total_trades / days
                    stats_text += f"📅 Trades per Day: {trades_per_day:.2f}\n"
                except ValueError:
                    pass

            await self._reply(update, stats_text.strip())

        except Exception as e:
            logger.error(f"Error in stats command: {e}")
            await self._reply(update, "❌ Error retrieving trading statistics.")
